%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% World Trade Model with Transport
% by Anders Stromman and Faye Duchin
%
% Programmed by
% Kristin Fjellheim
% October 21 2010
%
% This Matlab file calls GAMS and is a sequential procedure 
% for running the World Trade Model with Transportation 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Importing from GAMS

clear all
[VDFM,t,p,r] = gams('GTAP_Read_Vdfm.gms');                  % Domestic purchases by firms, at market prices
[VDGM,t,r] = gams('GTAP_Read_Vdgm.gms');                    % Domestic purchases by government, at market prices
[VDPM,t,r] = gams('GTAP_Read_Vdpm.gms');                    % Domestic purchases by households, at market prices
[VIFM,t,p,r] = gams('GTAP_Read_Vifm.gms');                  % Import purchases by firms, at market prices
[VIGM,t,r] = gams('GTAP_Read_Vigm.gms');                    % Import purchases by governments, at market prices
[VIPM,t,r] = gams('GTAP_Read_Vipm.gms');                    % Import purchases by households, at market prices
[TVOM,t,r] = gams('GTAP_Read_Tvom.gms');                    % Sales of domestic product, at market prices
[VFM,e,p,r] = gams('GTAP_Read_Vfm.gms');                    % Primary factor purchases by households, at market prices
[VIMS,t,r,rr] = gams('GTAP_Read_Vims.gms');                 % Imports, at market prices
[VXMD,t,r,rr] = gams('GTAP_Read_Vxmd.gms');                 % Exports, at market prices
[EVFA,e,p,r] = gams('GTAP_Read_Endw.gms');                  % Primary factor purchases, at agents prices
[EVOA,e,r] = gams('GTAP_Read_Evoa.gms');                    % Primary factor sales, at agents prices
[VTWR,m,t,r,rr] = gams('GTAP_Read_Vtwr.gms');               % Margins by margin commodity
[VST,m,r] = gams('GTAP_Read_Vst.gms');                      % Margin exports
[EGHG,ghg,e,s,r] = gams('GTAP_Read_CO2.gms');               % CO2 emissions
[NC_QO_CEQ,nco,p,r] = gams('GTAP_Read_NcQoCeq.gms');        % Non-CO2 emissions associated with output by industries
[NC_ENDW_CEQ,nco,e,p,r] = gams('GTAP_Read_NcEndwCeq.gms');  % Non-CO2 emissions associated with endowment by industries
[NC_TRAD_CEQ,nco,t,p,r] = gams('GTAP_Read_NcTradCeq.gms');  % Non-CO2 emissions associated with input use by industries
[NC_HH_CEQ,nco,t,r] = gams('GTAP_Read_NcHhCeq.gms');        % Non-CO2 emissions associated with input use by households

%% From structure to arrays

n_set = t.val;
p_set = p.val;
i_set = r.val;
k_set = e.val;
s_set = m.val;

%% Determining set sizes

n_max = size(n_set,1);   % goods + trsp (includes both goods and transport)
s_max = size(s_set,1);   % trsp 
k_max = size(k_set,1);   % factors 
i_max = size(i_set,1);   % regions

%% Converting structures to arrays for easier manipulation

Zd = VDFM.val(:,1:57,:);         % The Z matrices
yf = VDFM.val(:,end,:);         % Final demand from industries
yg = VDGM.val(:,:);             % Final demand from government
yh = VDPM.val(:,:);             % Final demand from housholds
Zim = VIFM.val(:,1:57,:);       % Imports by industries
Iyf = VIFM.val(:,end,:);        % Imports of final products by industries
Iyg = VIGM.val(:,:);            % Imports of final products by government
Iyh = VIPM.val(:,:);            % Imports of final products by households
g = TVOM.val(:,:);              % X vectors - Is the sum of exports, transport services, intermediate demand, investments, household consumpiton and government consumption
vfm = VFM.val(:,1:57,:);        % Primary factor purchases by households
vims = VIMS.val(:,:,:);         % Imports
vxmd = VXMD.val(:,:,:);         % Exports
evfa = EVFA.val(:,1:57,:);      % Value added
evoa = EVOA.val(:,:);           % Total endowments
vtwr = VTWR.val(:,:,:,:);       % Commodity exchange by transport type
vst = VST.val(:,:);             % margin exports
CO2 = EGHG.val(:,:,1:57,:);        % CO2 emissions
NCO2_qo = NC_QO_CEQ.val(:,1:57,:); % Stressors
NCO2_endw = NC_ENDW_CEQ.val(:,:,1:57,:); % Stressors
NCO2_trad = NC_TRAD_CEQ.val(:,:,1:57,:); % Stressors
NCO2_hh = NC_HH_CEQ.val(:,:,:); % Stressors
          
%% Creating Y vector

% Differentiating between domestic final demand and imported final demand
yf = reshape(yf,57,113);            % Reshaping final demand from industries
Iyf = reshape(Iyf,57,113);          % Reshaping final demand from imported products from industries

Yd = yf + yg + yh;                  % Total final demand on domestic products
Yim = Iyf + Iyg + Iyh;              % Total final demand on imported products

Y = Yd + Yim;                       % Total final demand

%% Creating A matrices

Z = Zd + Zim;

for r = 1:i_max
    for t = 1:n_max
        for tt = 1:n_max
            A(t,tt,r) = Zd(t,tt,r) / g(tt,r);
        end
    end
end

% Removing zeros from A-matrix
for t = 1:n_max
    for tt = 1:n_max
        for r = 1:i_max
            if A(t,tt,r) == 0;
                A(t,tt,r) = 9e10;
            end
        end
    end
end

%% Creating T matrices

T = zeros(57,57,113,113);

% Setting bulk goods equal to 20% of total cost where 80% occurs in water
% transport and 20% in land transport. Setting consumert goods equal to 5%
% of total cost, with distribution same as over. Setting transportation of
% transportation services, electricity and other administration services
% equal to 9e10 for no possibility of export. Importer pays for
% transportation.
for t = 1:n_max
    for tt = 1:n_max
        for r = 1:i_max
            for rr = 1:i_max
                T(48,1:26,r,rr) = 0.04;
                T(49,1:26,r,rr) = 0.16;
                T(48,32:42,r,rr) = 0.04;
                T(49,32:42,r,rr) = 0.16;
                T(48,27:31,r,rr) = 0.01;
                T(49,27:31,r,rr) = 0.04;
                T(t,43:57,r,rr) = 0.99;
                T(t,43:57,r,rr) = 0.99;
            end
        end
    end
end

%% Creating T matrices - 1st attempt
% vtwr shows the amount of transportation needed to transport good i from
% onoe region to another. vims shows the total import into a region.
% Dividing vtwr by vims gives the transportation requirements matrix. This
% means dividing transportation needed to transport everything of good i
% from region r to region rr by how much is being imported, gives the
% transportation requirements for one unit of that good.
% % for m = 1:s_max
% %     for t = 1:n_max
% %         for r = 1:i_max
% %             for rr = 1:i_max
% %                 T_tran(m,t,r,rr) = vtwr(m,t,r,rr) / vims(t,r,rr);
% %             end
% %         end
% %     end
% % end

% If divide by zero, set value equal to zero
% % T_tran(isnan(T_tran)) = 0;
% 
% Transforming T matrix into a 57 by 57 matrix with zeros everywhere but
% the transporation sectors
% % T = zeros(57,57,113,113);
% % T(48:50,:,:,:) = [T_tran(:,:,:,:)];

% Certain sectors are not able to transport. In these sectors a high number
% 9e10 will be put in to prevent it from being traded.
% % for t = 1:n_max
% %     for tt = 1:n_max
% %         for r = 1:i_max
% %             for rr = 1:i_max
% %                 T(t,1:3,r,rr) = 9e10;
% %                 T(t,9,r,rr) = 9e10;
% %                 T(t,14,r,rr) = 9e10;
% %                 T(t,46:57,r,rr) = 1e-10;
% %             end
% %         end
% %     end
% % end

%% Creating F matrices

for e = 1:k_max
    for t = 1:n_max
        for r = 1:i_max
            F(e,t,r) = vfm(e,t,r) / g(t,r);
        end
    end
end

% Removing zeros from F-matrix
for e = 1:k_max
    for t = 1:n_max
        for r = 1:i_max
            if F(e,t,r) == 0;
                F(e,t,r) = 9e10;
            end
        end
    end
end

%% Creating E vector (Total endowments)

%This is the sum of vfm (use when no factor constraint)
for r = 1:i_max
    E(:,r) = sum(vfm(:,:,r)')';
end

%% The total factor use

% Make it possible to increase the total factor use up to 3 times its
% initial value
FM = vfm;
% Not possible to remove all factor use, set minimum value to a reduction
% of 50%
FL = vfm;

%% Creating factor prices

% Assume currently that prices are equal to 1, in that way treating them as
% a exogenously variable
PI = ones(size(k_set,1), size(i_set,1));

%% Creating stressor matrix for CO2

% Reshaping the CO2 matrices that show what energy sector the emissions
% occur. Making it into a 57 vector with all the regions, divided by total
% production so as to get emission from production of one product.
g_sum = sum(g')';

% Reshaping it, since there is only one emission factor
CO2 = reshape(CO2,6,57,113);

% Adding it over all the energy sectors, so to get a 57*113 matrix.
% Reshaping
for r = 1:i_max
    for t = 1:n_max
        CO2_sum_sec(:,:,r) = sum(CO2(:,:,r));
    end
end
CO2_sum_sec = reshape(CO2_sum_sec,57,113);

% Adding it over all countries
CO2_sum = sum(CO2_sum_sec')';

% Finding emissions from production of one unit for each country
%S_CO2 = CO2_sum_sec./g;
S_CO2 = CO2_sum./g_sum;
%% Repeating above for non CO2 greenhouse gas emissions (CH4, N2O, Fgasses)

% For non-CO2 emissions associated with output by industries
qo_n2o = reshape(NCO2_qo(1,:,:),57,113);
qo_ch4 = reshape(NCO2_qo(2,:,:),57,113);
qo_fgas = reshape(NCO2_qo(3,:,:),57,113);

qo_n2o = sum(qo_n2o')';
qo_ch4 = sum(qo_ch4')';
qo_fgas = sum(qo_fgas')';

% For non-CO2 emissions associated with endowment by industries
endw_n2o = reshape(NCO2_endw(1,:,:,:),5,57,113);
endw_ch4 = reshape(NCO2_endw(2,:,:,:),5,57,113);
endw_fgas = reshape(NCO2_endw(3,:,:,:),5,57,113);

for r = 1:i_max
    endw_n2osum(:,:,r) = sum(endw_n2o(:,:,r));
    endw_ch4sum(:,:,r) = sum(endw_ch4(:,:,r));
    endw_fgassum(:,:,r) = sum(endw_fgas(:,:,r));
end
endw_n2o = reshape(endw_n2osum,57,113);
endw_ch4 = reshape(endw_ch4sum,57,113);
endw_fgas = reshape(endw_fgassum,57,113);

endw_n2o = sum(endw_n2o')';
endw_ch4 = sum(endw_ch4')';
endw_fgas = sum(endw_fgas')';

% For non-CO2 emissions associated with input use by industries
trad_n2o = reshape(NCO2_trad(1,:,:,:),57,57,113);
trad_ch4 = reshape(NCO2_trad(2,:,:,:),57,57,113);
trad_fgas = reshape(NCO2_trad(3,:,:,:),57,57,113);

for r = 1:i_max
    trad_n2osum(:,:,r) = sum(trad_n2o(:,:,r));
    trad_ch4sum(:,:,r) = sum(trad_ch4(:,:,r));
    trad_fgassum(:,:,r) = sum(trad_fgas(:,:,r));
end
trad_n2o = reshape(trad_n2osum,57,113);
trad_ch4 = reshape(trad_ch4sum,57,113);
trad_fgas = reshape(trad_fgassum,57,113);

trad_n2o = sum(trad_n2o')';
trad_ch4 = sum(trad_ch4')';
trad_fgas = sum(trad_fgas')';

% For non-CO2 emissions associated with input use by households
hh_n2o = reshape(NCO2_hh(1,:,:),57,113);
hh_ch4 = reshape(NCO2_hh(2,:,:),57,113);
hh_fgas = reshape(NCO2_hh(3,:,:),57,113);

hh_n2o = sum(hh_n2o')';
hh_ch4 = sum(hh_ch4')';
hh_fgas = sum(hh_fgas')';

% Adding the respective emissions together so as to get a total for N2O,
% CH4 and Fgasses
N2O_sum = qo_n2o + endw_n2o + trad_n2o + hh_n2o;
CH4_sum = qo_ch4 + endw_ch4 + trad_ch4 + hh_ch4;
FGAS_sum = qo_fgas + endw_fgas + trad_fgas + hh_fgas;

% Emission from production of one unit for each country
S_N2O = N2O_sum./g_sum;
S_CH4 = CH4_sum./g_sum;
S_FGAS = FGAS_sum./g_sum;

%% Total stressor matrix

S = [S_CO2,S_N2O,S_CH4,S_FGAS];